from typing import Any
import scrapy
from scrapy import Request
from scrapy import *
from . import scrapedo
import urllib.parse

class Spider(scrapy.Spider):
    def __init__(self, params: scrapedo.RequestParameters ,*args: Any, **kwargs: Any):
        self.params = params
        super().__init__(*args, **kwargs)
        

    def Request(self,*args: Any, **kwargs: Any) -> Request:
        return Request(self.params,*args, **kwargs)
    
    def origin_url(self,url: str) -> str:
        res = urllib.parse.urlparse(url)
        return urllib.parse.parse_qs(res.query)["url"][0]
    
    def target_url(self,response) -> str:
        return response.headers["Scrape.do-Target-Url"]
    
        
    
    

class Request(scrapy.Request):
    def __init__(self, params: scrapedo.RequestParameters, *args: Any, **kwargs: Any):
        
        url = kwargs.get("url")
        assert url is not None, "url is required"

        p = params.copy()
        p.params["url"] = url
        kwargs["url"] = p.encode()
        print(kwargs)
        super().__init__(*args, **kwargs)



import urllib.parse
import json


class RequestParameters:
    def __init__(self,token:str,url:str|None=None, params:dict=None) -> None:
        self.apiurl = "https://api.scrape.do/"
        self.token = token
        self.url = url
        self.params = {k.lower(): v for k, v in params.items()} if params else {}
        assert self.token is not None and self.token != "", 'Scrape.do token is required'
        


    def encode(self) -> str:
        if "playwithbrowser" in self.params and isinstance(self.params["playwithbrowser"],list):
            self.params["playwithbrowser"] = json.dumps(self.params["playwithbrowser"],indent="")
        
        if "url" not in self.params:
            # that means if params has url, it will use that one.
            self.params["url"] = self.url
        
        encoded_params = '&'.join(f'{urllib.parse.quote(str(k))}={urllib.parse.quote(str(v),safe="")}' for k, v in self.params.items())
        return f'{self.apiurl}?token={self.token}&{encoded_params}'

    def copy(self) -> 'RequestParameters':
        return RequestParameters(
            token=self.token,
            url=self.url,
            params=self.params.copy()
        )


if __name__ == "__main__":
    # small unit test
    print(RequestParameters(
        url="https://httpbin.co/",
        token="TOKEN",
        params={
            "render":True,
            "geoCode":"us",
            "playWithBrowser":[
                {
                    "Action": "Click",
                    "Selector":"#manpage > div.mp > ul > li:nth-child(3) > a"
                },
                {
                    "Action":"Wait",
                    "Timeout":2000,
                },{
                    "Action":"Execute",
                    "Execute":"document.URL",
                }
            ],
        },
    ).encode())

        




from setuptools import setup

setup(
    name='scrapy-scrapedo',
    version='0.1.3',
    url='https://github.com/scrape-do/scrapy-do',
    description='Fundemantal Scrapy support for Scrape.do API',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Scrape.do',
    author_email='hello@scrape.do',
    license='MIT',
    packages=['scrapydo'],
    classifiers=[ 
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Topic :: Software Development :: Libraries',
    'Topic :: Software Development :: Libraries :: Python Modules'
    ],  
    python_requires='>=3.8',
    install_requires=['scrapy'],
)
